@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -56,6 +56,8 @@ DEFAULT_CONFIGURATION = {
     'ssl_cert': None,
     'ssl_key': None,
     'ssl_verify_cert': False,
+    'ssl_cipher': None,
+    'ssl_disabled': False,
     'passwd': None,
     'db': None,
     'connect_timeout': None,
@@ -397,6 +399,8 @@ class ClientFlag(_Flags):
     CONNECT_ARGS = 1 << 20
     PLUGIN_AUTH_LENENC_CLIENT_DATA = 1 << 21
     CAN_HANDLE_EXPIRED_PASSWORDS = 1 << 22
+    SESION_TRACK = 1 << 23
+    DEPRECATE_EOF = 1 << 24
     SSL_VERIFY_SERVER_CERT = 1 << 30
     REMEMBER_OPTIONS = 1 << 31
 
@@ -419,6 +423,14 @@ class ClientFlag(_Flags):
         'SECURE_CONNECTION': (1 << 15, 'New 4.1 authentication'),
         'MULTI_STATEMENTS': (1 << 16, 'Enable/disable multi-stmt support'),
         'MULTI_RESULTS': (1 << 17, 'Enable/disable multi-results'),
+        'PS_MULTI_RESULTS': (1 << 18, 'Multi-results in PS-protocol'),
+        'PLUGIN_AUTH': (1 << 19, 'Client supports plugin authentication'),
+        'CONNECT_ARGS': (1 << 20, 'Client supports connection attributes'),
+        'PLUGIN_AUTH_LENENC_CLIENT_DATA': (1 << 21,
+                                           'Enable authentication response packet to be larger than 255 bytes'),
+        'CAN_HANDLE_EXPIRED_PASSWORDS': (1 << 22, "Don't close the connection for a connection with expired password"),
+        'SESION_TRACK': (1 << 23, 'Capable of handling server state change information'),
+        'DEPRECATE_EOF': (1 << 24, 'Client no longer needs EOF packet'),
         'SSL_VERIFY_SERVER_CERT': (1 << 30, ''),
         'REMEMBER_OPTIONS': (1 << 31, ''),
     }
@@ -461,6 +473,11 @@ class ServerFlag(_Flags):
     STATUS_LAST_ROW_SENT = 1 << 7
     STATUS_DB_DROPPED = 1 << 8
     STATUS_NO_BACKSLASH_ESCAPES = 1 << 9
+    SERVER_STATUS_METADATA_CHANGED = 1 << 10
+    SERVER_QUERY_WAS_SLOW = 1 << 11
+    SERVER_PS_OUT_PARAMS = 1 << 12
+    SERVER_STATUS_IN_TRANS_READONLY = 1 << 13
+    SERVER_SESSION_STATE_CHANGED = 1 << 14
 
     desc = {
         'SERVER_STATUS_IN_TRANS': (1 << 0,
@@ -472,10 +489,31 @@ class ServerFlag(_Flags):
                                        'next query exists'),
         'SERVER_QUERY_NO_GOOD_INDEX_USED': (1 << 4, ''),
         'SERVER_QUERY_NO_INDEX_USED': (1 << 5, ''),
-        'SERVER_STATUS_CURSOR_EXISTS': (1 << 6, ''),
-        'SERVER_STATUS_LAST_ROW_SENT': (1 << 7, ''),
+        'SERVER_STATUS_CURSOR_EXISTS': (1 << 6,
+                                        'Set when server opened a read-only '
+                                        'non-scrollable cursor for a query.'),
+        'SERVER_STATUS_LAST_ROW_SENT': (1 << 7,
+                                        'Set when a read-only cursor is '
+                                        'exhausted'),
         'SERVER_STATUS_DB_DROPPED': (1 << 8, 'A database was dropped'),
         'SERVER_STATUS_NO_BACKSLASH_ESCAPES': (1 << 9, ''),
+        'SERVER_STATUS_METADATA_CHANGED': (1024,
+                                           'Set if after a prepared statement '
+                                           'reprepare we discovered that the '
+                                           'new statement returns a different '
+                                           'number of result set columns.'),
+        'SERVER_QUERY_WAS_SLOW': (2048, ''),
+        'SERVER_PS_OUT_PARAMS': (4096,
+                                 'To mark ResultSet containing output '
+                                 'parameter values.'),
+        'SERVER_STATUS_IN_TRANS_READONLY': (8192,
+                                            'Set if multi-statement '
+                                            'transaction is a read-only '
+                                            'transaction.'),
+        'SERVER_SESSION_STATE_CHANGED': (1 << 14,
+                                         'Session state has changed on the '
+                                         'server because of the execution of '
+                                         'the last statement'),
     }
 
 
@@ -675,7 +713,7 @@ class CharacterSet(_Constants):
         return tuple(res)
 
 
-class SQLMode(_Constants):  # pylint: disable=R0921
+class SQLMode(_Constants):
     """MySQL SQL Modes
 
     The numeric values of SQL Modes are not interesting, only the names
