@@ -1,5 +1,5 @@
 # MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
 
 # MySQL Connector/Python is licensed under the terms of the GPLv2
 # <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -29,8 +29,9 @@ import re
 import weakref
 
 from . import errors
-from .abstracts import MySQLCursorAbstract
+from .abstracts import MySQLCursorAbstract, NAMED_TUPLE_CACHE
 from .catch23 import PY2
+from .constants import ServerFlag
 
 SQL_COMMENT = r"\/\*.*?\*\/"
 RE_SQL_COMMENT = re.compile(
@@ -40,7 +41,8 @@ RE_SQL_ON_DUPLICATE = re.compile(
     r'''\s*ON\s+DUPLICATE\s+KEY(?:[^"'`]*["'`][^"'`]*["'`])*[^"'`]*$''',
     re.I | re.M | re.S)
 RE_SQL_INSERT_STMT = re.compile(
-    r"({0}|\s)*INSERT({0}|\s)*INTO.+VALUES.*".format(SQL_COMMENT),
+    r"({0}|\s)*INSERT({0}|\s)*INTO\s+[`'\"]?.+[`'\"]?(?:\.[`'\"]?.+[`'\"]?)"
+    r"{{0,2}}\s+VALUES\s*\(.+(?:\s*,.+)*\)".format(SQL_COMMENT),
     re.I | re.M | re.S)
 RE_SQL_INSERT_VALUES = re.compile(r'.*VALUES\s*(\(.*\)).*', re.I | re.M | re.S)
 RE_PY_PARAM = re.compile(b'(%s)')
@@ -59,6 +61,7 @@ RE_SQL_FIND_PARAM = re.compile(
 
 ERR_NO_RESULT_TO_FETCH = "No result set to fetch from"
 
+MAX_RESULTS = 4294967295
 
 class _ParamSubstitutor(object):
     """
@@ -96,20 +99,27 @@ def _bytestr_format_dict(bytestr, value_dict):
     b'x=%(y)s y=%(x)s'
     """
     def replace(matchobj):
+        """Replace pattern."""
         value = None
         groups = matchobj.groupdict()
         if groups["conversion_type"] == b"%":
             value = b"%"
         if groups["conversion_type"] == b"s":
-            key = groups["mapping_key"].encode("utf-8") \
-                  if PY2 else groups["mapping_key"]
+            key = groups["mapping_key"]
             value = value_dict[key]
         if value is None:
             raise ValueError("Unsupported conversion_type: {0}"
                              "".format(groups["conversion_type"]))
-        return value.decode("utf-8") if PY2 else value
-    return RE_PY_MAPPING_PARAM.sub(replace, bytestr.decode("utf-8")
-                                   if PY2 else bytestr)
+        return bytes(value) if PY2 else value
+
+    stmt = RE_PY_MAPPING_PARAM.sub(replace, bytestr)
+    if PY2:
+        try:
+            return stmt.decode("utf-8")
+        except UnicodeDecodeError:
+            pass
+    return stmt
+
 
 class CursorBase(MySQLCursorAbstract):
     """
@@ -579,11 +589,11 @@ class MySQLCursor(CursorBase):
                 "Failed rewriting statement for multi-row INSERT. "
                 "Check SQL syntax."
             )
-        fmt = matches.group(1).encode(self._connection.charset)
+        fmt = matches.group(1).encode(self._connection.python_charset)
         values = []
 
         try:
-            stmt = operation.encode(self._connection.charset)
+            stmt = operation.encode(self._connection.python_charset)
             for params in seq_params:
                 tmp = fmt
                 if isinstance(params, dict):
@@ -748,10 +758,12 @@ class MySQLCursor(CursorBase):
             can_consume_results = self._connection._consume_results
             for result in self._connection.cmd_query_iter(call):
                 self._connection._consume_results = False
+                # pylint: disable=R0204
                 if self._raw:
                     tmp = MySQLCursorBufferedRaw(self._connection._get_self())
                 else:
                     tmp = MySQLCursorBuffered(self._connection._get_self())
+                # pylint: enable=R0204
                 tmp._executed = "(a result of {0})".format(call)
                 tmp._handle_result(result)
                 if tmp._warnings is not None:
@@ -759,7 +771,7 @@ class MySQLCursor(CursorBase):
                 if 'columns' in result:
                     results.append(tmp)
             self._connection._consume_results = can_consume_results
-            #pylint: enable=W0212
+            # pylint: enable=W0212
 
             if argnames:
                 select = "SELECT {0}".format(','.join(argtypes))
@@ -1070,6 +1082,36 @@ class MySQLCursorPrepared(MySQLCursor):
         self._prepared = None
         self._binary = True
         self._have_result = None
+        self._last_row_sent = False
+        self._cursor_exists = False
+
+    def reset(self, free=True):
+        if self._prepared:
+            try:
+                self._connection.cmd_stmt_close(self._prepared['statement_id'])
+            except errors.Error:
+                # We tried to deallocate, but it's OK when we fail.
+                pass
+            self._prepared = None
+        self._last_row_sent = False
+        self._cursor_exists = False
+
+    def _handle_noresultset(self, res):
+        self._handle_server_status(res.get('status_flag',
+                                           res.get('server_status', 0)))
+        super(MySQLCursorPrepared, self)._handle_noresultset(res)
+
+    def _handle_server_status(self, flags):
+        """Check for SERVER_STATUS_CURSOR_EXISTS and
+           SERVER_STATUS_LAST_ROW_SENT flags set by the server.
+        """
+        self._cursor_exists = flags & ServerFlag.STATUS_CURSOR_EXISTS != 0
+        self._last_row_sent = flags & ServerFlag.STATUS_LAST_ROW_SENT != 0
+
+    def _handle_eof(self, eof):
+        self._handle_server_status(eof.get('status_flag',
+                                           eof.get('server_status', 0)))
+        super(MySQLCursorPrepared, self)._handle_eof(eof)
 
     def callproc(self, *args, **kwargs):
         """Calls a stored procedue
@@ -1084,13 +1126,7 @@ class MySQLCursorPrepared(MySQLCursor):
         This method will try to deallocate the prepared statement and close
         the cursor.
         """
-        if self._prepared:
-            try:
-                self._connection.cmd_stmt_close(self._prepared['statement_id'])
-            except errors.Error:
-                # We tried to deallocate, but it's OK when we fail.
-                pass
-            self._prepared = None
+        self.reset()
         super(MySQLCursorPrepared, self).close()
 
     def _row_to_python(self, rowdata, desc=None):
@@ -1112,6 +1148,11 @@ class MySQLCursorPrepared(MySQLCursor):
             self._connection.unread_result = True
             self._have_result = True
 
+            if 'status_flag' in res[2]:
+                self._handle_server_status(res[2]['status_flag'])
+            elif 'server_status' in res[2]:
+                self._handle_server_status(res[2]['server_status'])
+
     def execute(self, operation, params=(), multi=False):  # multi is unused
         """Prepare and execute a MySQL Prepared Statement
 
@@ -1189,6 +1230,8 @@ class MySQLCursorPrepared(MySQLCursor):
 
         Returns a tuple or None.
         """
+        if self._cursor_exists:
+            self._connection.cmd_stmt_fetch(self._prepared['statement_id'])
         return self._fetch_row() or None
 
     def fetchmany(self, size=None):
@@ -1204,10 +1247,18 @@ class MySQLCursorPrepared(MySQLCursor):
     def fetchall(self):
         if not self._have_unread_result():
             raise errors.InterfaceError("No result set to fetch from.")
-        (rows, eof) = self._connection.get_rows(
-            binary=self._binary, columns=self.description)
+        rows = []
+        if self._nextrow[0]:
+            rows.append(self._nextrow[0])
+        while self._have_unread_result():
+            if self._cursor_exists:
+                self._connection.cmd_stmt_fetch(
+                    self._prepared['statement_id'], MAX_RESULTS)
+            (tmp, eof) = self._connection.get_rows(
+                binary=self._binary, columns=self.description)
+            rows.extend(tmp)
+            self._handle_eof(eof)
         self._rowcount = len(rows)
-        self._handle_eof(eof)
         return rows
 
 
@@ -1283,9 +1334,14 @@ class MySQLCursorNamedTuple(MySQLCursor):
 
         if row:
             # pylint: disable=W0201
-            self.named_tuple = namedtuple('Row', self.column_names)
+            columns = tuple(self.column_names)
+            try:
+                named_tuple = NAMED_TUPLE_CACHE[columns]
+            except KeyError:
+                named_tuple = namedtuple('Row', columns)
+                NAMED_TUPLE_CACHE[columns] = named_tuple
             # pylint: enable=W0201
-            return self.named_tuple(*row)
+            return named_tuple(*row)
 
     def fetchone(self):
         """Returns next row of a query result set
